/*
 * Project_name:
    StepperMotor
 * Copyright:
     (c) mikroElektronika, 2011.
 * Revision History
    20101102:
      - Initial release;
    20101102:
      - modifications by RR;
    20110618:
      - modified by SZ (sbit approach, code reorganized, comments updated);
 * Description:
     This is a simple demonstration of the Bipolar Stepper Board usage. You can toggle the stepper motor resolution
     and initiate CW or CCW motor rotations. Monitor resolution and rotations at LCD (also PORT3 LEDs represent the
     stepper motor resolution).
 * Test configuration:
     MCU:             AT89S8253
                      http://www.atmel.com/dyn/resources/prod_documents/doc3286.pdf
     Dev.Board:       Easy8051v6
                      http://www.mikroe.com/eng/products/view/329/easy8051-v6-development-system/
     Oscillator:      External Clock 10.0000 MHz
     Ext. Modules:    Bipolar Stepper Motor Driver Board at PORT1
                      http://www.mikroe.com/eng/products/view/217/bipolar-stepper-motor-driver-board/
     SW:              mikroC PRO for 8051
                      http://www.mikroe.com/eng/products/view/232/mikroc-pro-for-8051/
 * NOTES:
  - Pull-up  PORT1 (SW2) and connect Bipolar Stepper Motor Driver Board to PORT1
  - Bipolar stepper motor connections:
                  OUT1B - Red wire
                  OUT1A - Blue wire
                  OUT2B - Yellow wire
                  OUT2A - White wire
  - Turn on LCD backlight switch
  - PORT3 LEDS - ON (SW7.4); Pull up PORT3(SW4)
  - Pull-up PORT0 (P0.0, P0.1, P0.2 and P0.3) - SW1.1, SW1.2, SW1.3 and SW1.4 and J16 - Gnd
*/

// Bipolar Stepper motor driver connections
sbit DIR      at   P1_6_bit;
sbit RESET_   at   P1_2_bit;
sbit ENABLE_  at   P1_0_bit;
sbit STEP     at   P1_1_bit;
sbit SLEEP_   at   P1_3_bit;
sbit MS1      at   P1_4_bit;
sbit MS2      at   P1_5_bit;
//

// Lcd module connections
sbit LCD_RS at P2_0_bit;
sbit LCD_EN at P2_1_bit;

sbit LCD_D4 at P2_2_bit;
sbit LCD_D5 at P2_3_bit;
sbit LCD_D6 at P2_4_bit;
sbit LCD_D7 at P2_5_bit;
// End Lcd module connections

char cnt = 0;
char resolution = 1;

void Step_x1(){
 STEP = 0;
 Delay_ms(5);
 STEP = 1;
 Delay_ms(5);
}


void main() {


  DIR = 0;
  RESET_ = 1;
  ENABLE_ = 1;
  STEP = 0;
  SLEEP_ = 1;
  MS1 = 0;
  MS2 = 0;

  Lcd_Init();                        // Init LCD

  Lcd_Cmd(_LCD_CLEAR);               // Clear display
  Lcd_Cmd(_LCD_CURSOR_OFF);          // Cursor off

  Lcd_Out(1, 1, "Res: ");
  Lcd_Out(2, 1, "Rotation: ");

  while (1) {

//       MS2           MS1             Resolution
//        0             0                  1
//        0             1                 1/2
//        1             0                 1/4
//        1             1                 1/8


    if (!P0_3_bit) {
                   Delay_ms(100);
                   if (!P0_3_bit) resolution++;
                   }
    if(resolution > 4) resolution = 4;


    if (!P0_0_bit) {
                   Delay_ms(100);
                   if (!P0_0_bit) resolution--;
                   }
    if (resolution < 1) resolution = 1;


          switch (resolution) {
            case 1:
              MS1 = 0;
              MS2 = 0;
              Lcd_Out(1, 7, "Full Step");
              P3 = 0b00000000;
              break;
            case 2:
              MS1 = 1;
              MS2 = 0;
              Lcd_Out(1, 7, "1/2 Step ");
              P3 = 0b11110000;
              break;
            case 3:
              MS1 = 0;
              MS2 = 1;
              Lcd_Out(1, 7, "1/4 Step ");
              P3 = 0b11111100;
              break;
            case 4:
              MS1 = 1;
              MS2 = 1;
              Lcd_Out(1, 7, "1/8 Step ");
              P3 = 0b11111110;
              break;
          }


  // Rotate CW
    if (!P0_1_bit) {
                   Delay_ms(100);
                      if (!P0_1_bit) {
                          Lcd_Out(2, 11, " CW");

                          DIR = 0;
                          ENABLE_ = 0;

                          for (cnt = 0; cnt <= 200; cnt++)  Step_x1();

                          Delay_ms(300);
                          Lcd_Out(2, 11, "     ");
                          ENABLE_ = 1;
                   }
    }


  // Rotate CCW
    if (!P0_2_bit) {
                   Delay_ms(100);
                      if (!P0_2_bit) {
                        Lcd_Out(2, 11, " CCW");

                        DIR = 1;
                        ENABLE_ = 0;

                        for (cnt = 0; cnt <= 200; cnt++) Step_x1();

                        Delay_ms(300);
                        Lcd_Out(2, 11, "     ");
                        ENABLE_ = 1;
                      }
    }

  }
}
