/*
 * Project_name:
    StepperMotor
 * Copyright:
     (c) mikroElektronika, 2011.
 * Revision History
    20101102:
      - Initial release;
    20101102:
      - modifications by RR;
    20110618:
      - modified by SZ (sbit approach, code reorganized, comments updated);
 * Description:
     This is a simple demonstration of the Bipolar Stepper Board usage. Use Up and Down arrow buttons
     to toggle stepper motor resolution. Use Left and Right arrow buttons to initiate CW or CCW motor rotations.
     You can monitor resolution and rotations at LCD (also PORTC LEDs represent the stepper motor resolution).
     MCU:             ATmega16
                      http://www.atmel.com/dyn/resources/prod_documents/doc2466.pdf
     Dev.Board:       EasyAVR6
                      http://www.mikroe.com/eng/products/view/321/easyavr6-development-system/
     Oscillator:      External Clock 08.0000 MHz
     Ext. Modules:    ac:Bipolar_Stepper_Motor_Driver_Board at PORTD
                      http://www.mikroe.com/eng/products/view/217/bipolar-stepper-motor-driver-board/
     SW:              mikroC PRO for AVR
                      http://www.mikroe.com/eng/products/view/228/mikroc-pro-for-avr/
 * NOTES:
  - Pull-down PORTD (SW4) and connect Bipolar Stepper Motor Driver Board to PORTD
  - Bipolar stepper motor connections:
                  OUT1B - Red wire
                  OUT1A - Blue wire
                  OUT2B - Yellow wire
                  OUT2A - White wire
  - PORTC LEDS - ON (SW8.3)
  - Pull down PORTA (PA0, PA1, PA2 and PA3) - SW1.1, SW1.2, SW1.3 and SW1.4 and J17 - Vcc
  - Turn on switches SW9.7, SW9.8, SW6.1, SW6.5, SW6.7, SW7.1 and SW7.5.
  - Turn on COG LCD 2x16 switches SW10.1, SW10.2, SW10.3, SW10.4, SW10.5 and SW10.6.
*/

// Port Expander module connections
sbit  SPExpanderRST at PORTB2_bit;
sbit  SPExpanderCS  at PORTB1_bit;
sbit  SPExpanderRST_Direction at DDB2_bit;
sbit  SPExpanderCS_Direction  at DDB1_bit;
// End Port Expander module connections


// Bipolar Stepper motor driver connections
sbit DIR      at   PORTD6_bit;
sbit RESET_   at   PORTD2_bit;
sbit ENABLE_  at   PORTD0_bit;
sbit STEP     at   PORTD1_bit;
sbit SLEEP_   at   PORTD3_bit;
sbit MS1      at   PORTD4_bit;
sbit MS2      at   PORTD5_bit;

sbit DIR_Direction    at   DDD6_bit;
sbit RESET_Direction  at   DDD2_bit;
sbit ENABLE_Direction at   DDD0_bit;
sbit STEP_Direction   at   DDD1_bit;
sbit SLEEP_Direction  at   DDD3_bit;
sbit MS1_Direction    at   DDD4_bit;
sbit MS2_Direction    at   DDD5_bit;
//

char cnt = 0;
char resolution = 1;

void Step_x1(){
 STEP = 0;
 Delay_ms(5);
 STEP = 1;
 Delay_ms(5);
}


void main() {

  DIR = 0;
  RESET_ = 1;
  ENABLE_ = 1;
  STEP = 0;
  SLEEP_ = 1;
  MS1 = 0;
  MS2 = 0;

  DIR_Direction = 1;
  RESET_Direction = 1;
  ENABLE_Direction = 1;
  STEP_Direction = 1;
  SLEEP_Direction = 1;
  MS1_Direction = 1;
  MS2_Direction = 1;

// Control buttons
  DDA0_bit = 0;
  DDA1_bit = 0;
  DDA2_bit = 0;
  DDA3_bit = 0;
//

  DDRC = 0xFF;                           // Set PortC as output
  PORTC = 0x00;

  // Port Expander Library uses SPI1 module
  SPI1_Init();                           // Initialize SPI module used with PortExpander

  SPI_Lcd_Config(0);                     // Initialize Lcd over SPI interface
  SPI_Lcd_Cmd(_LCD_CLEAR);               // Clear display
  SPI_Lcd_Cmd(_LCD_CURSOR_OFF);          // Turn cursor off

  SPI_Lcd_Out(1, 1, "Res: ");
  SPI_Lcd_Out(2, 1, "Rotation: ");

  while (1) {

//     MS2(PD.5)     MS1(PD.4)          Resolution
//        0             0                  1
//        0             1                 1/2
//        1             0                 1/4
//        1             1                 1/8


    if (Button(&PINA, 3, 100, 1)) resolution++;
    if(resolution > 4) resolution = 4;


    if (Button(&PINA, 0, 100, 1)) resolution--;
    if (resolution < 1) resolution = 1;


          switch (resolution) {
            case 1:
              MS1 = 0;
              MS2 = 0;
              SPI_Lcd_Out(1, 7, "Full Step");
              PORTC = 0b11111111;
              break;
            case 2:
              MS1 = 1;
              MS2 = 0;
              SPI_Lcd_Out(1, 7, "1/2 Step ");
              PORTC = 0b00001111;
              break;
            case 3:
              MS1 = 0;
              MS2 = 1;
              SPI_Lcd_Out(1, 7, "1/4 Step ");
              PORTC = 0b00000011;
              break;
            case 4:
              MS1 = 1;
              MS2 = 1;
              SPI_Lcd_Out(1, 7, "1/8 Step ");
              PORTC = 0b00000001;
              break;
          }


  // Rotate CW
    if (Button(PINA, 1, 50, 1)){
      SPI_Lcd_Out(2, 11, " CW");

      DIR = 0;
      ENABLE_ = 0;

      for (cnt = 0; cnt <= 200; cnt++)  Step_x1();

      Delay_ms(300);
      SPI_Lcd_Out(2, 11, "     ");
      ENABLE_ = 1;
    }


  // Rotate CCW
    if (Button(&PINA, 2, 50, 1)){
      SPI_Lcd_Out(2, 11, " CCW");

      DIR = 1;
      ENABLE_ = 0;

      for (cnt = 0; cnt <= 200; cnt++) Step_x1();

      Delay_ms(300);
      SPI_Lcd_Out(2, 11, "     ");
      ENABLE_ = 1;
    }
  }
}