/*
 * Project_name:
    StepperMotor
 * Copyright:
     (c) mikroElektronika, 2011.
 * Revision History
    20101102:
      - Initial release;
    20101102:
      - modifications by RR;
    20110615:
      - modified by SZ (sbit approach, code reorganized, comments updated);
 * Description:
     This is a simple demonstration of the Bipolar Stepper Board usage. Use Up and Down arrow buttons
     to toggle stepper motor resolution. Use Left and Right arrow buttons to initiate CW or CCW motor rotations.
     You can monitor resolution and rotations at LCD (also PORTC LEDs represent the stepper motor resolution).
 * Test configuration:
     MCU:                 ac:PIC16F887
                          http://ww1.microchip.com/downloads/en/DeviceDoc/41291F.pdf
     Dev.Board:           ac:EasyPIC6
                          http://www.mikroe.com/en/tools/easypic6/
     Oscillator:          HS, 8.0000 MHz
     Ext. Modules:        ac:Bipolar_Stepper_Motor_Driver_Board at PORTD
                          http://www.mikroe.com/eng/products/view/217/bipolar-stepper-motor-driver-board/
     SW:                  mikroC PRO for PIC
                          http://www.mikroe.com/eng/products/view/7/mikroc-pro-for-pic/
 * NOTES:
  - Pull-down PORTD (SW4) and connect Bipolar Stepper Motor Driver Board to PORTD
  - Bipolar stepper motor connections:
                  OUT1B - Red wire
                  OUT1A - Blue wire
                  OUT2B - Yellow wire
                  OUT2A - White wire
  - Turn On LCD backlight (SW6.8)
  - PORTC LEDS - ON (SW9.3)
  - Pull down PORTA (RA0, RA1, RA2 and RA3) - SW1.1, SW1.2, SW1.3 and SW1.4 and J17 - Vcc

*/

// Bipolar Stepper motor driver connections
sbit DIR      at   RD6_bit;
sbit RESET_   at   RD2_bit;
sbit ENABLE_  at   RD0_bit;
sbit STEP     at   RD1_bit;
sbit SLEEP_   at   RD3_bit;
sbit MS1      at   RD4_bit;
sbit MS2      at   RD5_bit;

sbit DIR_Direction    at   TRISD6_bit;
sbit RESET_Direction  at   TRISD2_bit;
sbit ENABLE_Direction at   TRISD0_bit;
sbit STEP_Direction   at   TRISD1_bit;
sbit SLEEP_Direction  at   TRISD3_bit;
sbit MS1_Direction    at   TRISD4_bit;
sbit MS2_Direction    at   TRISD5_bit;
//

// LCD module connections
sbit LCD_RS at RB4_bit;
sbit LCD_EN at RB5_bit;
sbit LCD_D4 at RB0_bit;
sbit LCD_D5 at RB1_bit;
sbit LCD_D6 at RB2_bit;
sbit LCD_D7 at RB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End LCD module connections
  
char cnt = 0;
char resolution = 1;

void Step_x1(){
 STEP = 0;
 Delay_ms(5);
 STEP = 1;
 Delay_ms(5);
}


void main() {

  ANSEL = 0x00;                      // Configure all pins as Digital I/Os
  ANSELH = 0x00;
  C1ON_bit = 0;                      // Disable comparators
  C2ON_bit = 0;

  DIR = 0;
  RESET_ = 1;
  ENABLE_ = 1;
  STEP = 0;
  SLEEP_ = 1;
  MS1 = 0;
  MS2 = 0;

  DIR_Direction = 0;
  RESET_Direction = 0;
  ENABLE_Direction = 0;
  STEP_Direction = 0;
  SLEEP_Direction = 0;
  MS1_Direction = 0;
  MS2_Direction = 0;

  TRISA = 0x0F;                      // Keypad inputs

  TRISC = 0x00;                      // Set PortC as output
  PORTC = 0x00;

  Lcd_Init();                        // Init LCD

  Lcd_Cmd(_LCD_CLEAR);               // Clear display
  Lcd_Cmd(_LCD_CURSOR_OFF);          // Cursor off
  
  Lcd_Out(1, 1, "Res: ");
  Lcd_Out(2, 1, "Rotation: ");

  while (1) {

//      MS2(RD5_bit)    MS1(RD4_bit)    Resolution
//        0             0                  1
//        0             1                 1/2
//        1             0                 1/4
//        1             1                 1/8


    if (Button(&PORTA, 3, 100, 1)) resolution++;
    if(resolution > 4) resolution = 4;


    if (Button(&PORTA, 0, 100, 1)) resolution--;
    if (resolution < 1) resolution = 1;
         

          switch (resolution) {
            case 1:
              MS1 = 0;
              MS2 = 0;
              Lcd_Out(1, 7, "Full Step");
              PORTC = 0b11111111;
              break;
            case 2:
              MS1 = 1;
              MS2 = 0;
              Lcd_Out(1, 7, "1/2 Step ");
              PORTC = 0b00001111;
              break;
            case 3:
              MS1 = 0;
              MS2 = 1;
              Lcd_Out(1, 7, "1/4 Step ");
              PORTC = 0b00000011;
              break;
            case 4:
              MS1 = 1;
              MS2 = 1;
              Lcd_Out(1, 7, "1/8 Step ");
              PORTC = 0b00000001;
              break;
          }


  // Rotate CW
    if (Button(&PORTA, 1, 50, 1)){
      Lcd_Out(2, 11, " CW");

      DIR = 0;
      ENABLE_ = 0;

      for (cnt = 0; cnt <= 200; cnt++)  Step_x1();

      Delay_ms(300);
      Lcd_Out(2, 11, "     ");
      ENABLE_ = 1;
    }


  // Rotate CCW
    if (Button(&PORTA, 2, 50, 1)){
      Lcd_Out(2, 11, " CCW");

      DIR = 1;
      ENABLE_ = 0;

      for (cnt = 0; cnt <= 200; cnt++) Step_x1();

      Delay_ms(300);
      Lcd_Out(2, 11, "     ");
      ENABLE_ = 1;
    }
  }
}